/*
 * Copyright (c) 2008 BEA Systems, Inc. All Rights Reserved.
 *
 * $Id: //Dev/ucmcrawler/main/src/com/oracle/wci/ucm/Provider.java#3 $
 *
 * This software is the confidential and proprietary information of BEA Systems, Inc. ("Confidential
 * Information"). You shall not disclose such Confidential Information and shall use it only in accordance
 * with the terms of the license agreement you entered into with BEA Systems Inc.
 */

package com.oracle.wci.ucm;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.WeakHashMap;

import oracle.stellent.ridc.IdcClient;
import oracle.stellent.ridc.IdcClientException;
import oracle.stellent.ridc.IdcClientManager;
import oracle.stellent.ridc.IdcContext;
import oracle.stellent.ridc.model.DataBinder;
import oracle.stellent.ridc.model.DataObject;
import oracle.stellent.ridc.model.DataResultSet;
import oracle.stellent.ridc.protocol.ServiceResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.plumtree.remote.ServiceException;
import com.plumtree.remote.crawler.ACLEntry;
import com.plumtree.remote.crawler.CrawlerInfo;
import com.plumtree.remote.crawler.ICrawlerLog;
import com.plumtree.remote.util.DataSourceInfo;

/**
 * Methods common to DocumentProvider and ContainerProvider.
 * 
 * @author <a href="mailto:clinton.davidson@oracle.com">Clinton Davidson</a>
 * @version $Revision: #3 $ ($Change: 348620 $)
 */
public class Provider {
	/**
	 * Table name key to query role definition table.
	 */
	private static final String TABLE_NAME = "tableName";
	/**
	 * Name of group column in role definition table.
	 */
	private static final String D_GROUP_NAME = "dGroupName";
	/**
	 * Name of role column in role definition table.
	 */
	private static final String D_ROLE_NAME = "dRoleName";
	/**
	 * Name of table that contains role-group mappings.
	 */
	private static final String ROLE_DEFINITION = "RoleDefinition";
	/**
	 * Service name to get the resultset from a table.
	 */
	private static final String GET_TABLE = "GET_TABLE";

	/**
	 * Apache Commons Logger.
	 */
	private static final Log LOGGER = LogFactory.getLog(Provider.class);

	/**
	 * NamedValue settings from the crawler.
	 */
	private CrawlerInfo crawlerInfo;
	/**
	 * NamedValue settings from the data source.
	 */
	private DataSourceInfo dataInfo;

	/**
	 * CWS logger- available to Containers but not Documents..
	 */
	private ICrawlerLog logger;

	/**
	 * Sets the NamedValue settings from the crawler.
	 * 
	 * @param crawlInfo
	 *            NamedValue settings from the crawler
	 */
	public final void setCrawlerInfo(CrawlerInfo crawlInfo) {
		this.crawlerInfo = crawlInfo;
	}

	/**
	 * Gets the NamedValue settings from the crawler.
	 * 
	 * @return NamedValue settings from the crawler
	 */
	public final CrawlerInfo getCrawlerInfo() {
		return this.crawlerInfo;
	}

	/**
	 * Map of security group names and matching entries.
	 */
	private Map<String, Set<ACLEntry>> securityGroupEntriesMap = new WeakHashMap<String, Set<ACLEntry>>();

	/**
	 * Gets the roles from the security group. As we don't need to make this
	 * call more than necessary, cache the entries and clear them during
	 * initialize calls.
	 * 
	 * @param securityGroupName
	 *            name of the security group
	 * @return set of roles corresponding to the specified security group name
	 */
	public Set<ACLEntry> getRolesFromSecurityGroup(String securityGroupName) {
		Set aclEntrySet = new HashSet<ACLEntry>();
		// if there is an existing entry- use it. We are not concerned about
		// roles changing in the middle of a crawl.
		synchronized (securityGroupEntriesMap) {
			aclEntrySet = securityGroupEntriesMap.get(securityGroupName);
			if (null != aclEntrySet) {
				return aclEntrySet;
			}
		}
		// entries is null at this point, so call UCM to get entries info
		final String idcUrl = getIdcUrl();
		final String username = getIdcUsername();
		final IdcClientManager idcManager = new IdcClientManager();
		try {
			final IdcClient idcClient = idcManager.createClient(idcUrl);
			final IdcContext idcContext = new IdcContext(username);
			final DataBinder urlBinder = idcClient.createBinder();
			// get the role definition table, which lists each role mapped to
			// each security group
			urlBinder.putLocal(UcmConstants.IDC_SERVICE, GET_TABLE);
			urlBinder.putLocal(TABLE_NAME, ROLE_DEFINITION);
			final ServiceResponse urlResponse = idcClient.sendRequest(idcContext, urlBinder);
			final DataBinder urlResponseBinder = urlResponse.getResponseAsBinder();
			final DataResultSet rolesResultSet = urlResponseBinder.getResultSet(ROLE_DEFINITION);
			final List<DataObject> rows = rolesResultSet.getRows();
			synchronized (securityGroupEntriesMap) {
				for (DataObject row : rows) {
					// make an entry
					final String roleName = row.get(D_ROLE_NAME);
					final ACLEntry entry = new ACLEntry();
					entry.setDomain(CrawlConstants.Misc.DOMAIN_NAME);
					entry.setName(roleName);
					// don't worry about privilege, as we only care about read
					// access

					final String groupName = row.get(D_GROUP_NAME);
					Set<ACLEntry> entrySet = securityGroupEntriesMap.get(groupName);
					if (null == entrySet) {
						entrySet = new HashSet<ACLEntry>();
					}
					entrySet.add(entry);

					// add the set to the map
					securityGroupEntriesMap.put(groupName, entrySet);
				}
			}

		}
		catch (IdcClientException e) {
			LOGGER.error(e.getMessage(), e);
			return new HashSet<ACLEntry>();
		}

		// get the entry, and return an empty list if there is no match (which
		// should never happen)
		synchronized (securityGroupEntriesMap) {
			aclEntrySet = securityGroupEntriesMap.get(securityGroupName);
			if (null == aclEntrySet) {
				aclEntrySet = new HashSet<ACLEntry>();
			}
			return aclEntrySet;
		}
	}

	/**
	 * Clears the security group to role map. Done on shutdown for both
	 * providers. While there may be a more efficient way to do this, we want to
	 * avoid keeping items in a cache when UCM has changed them in the meantime.
	 * We are not concerned about changes during a crawl.
	 */
	public void clearSecurityGroupRoleMap() {
		synchronized (securityGroupEntriesMap) {
			securityGroupEntriesMap.clear();
		}
	}

	/**
	 * Sets the NamedValue settings from the data source.
	 * 
	 * @param dataSourceInfo
	 *            the NamedValue settings from the data source.
	 */
	public final void setDataSourceInfo(DataSourceInfo dataSourceInfo) {
		this.dataInfo = dataSourceInfo;
	}

	/**
	 * Gets the the NamedValue settings from the data source.
	 * 
	 * @return the NamedValue settings from the data source.
	 */
	public final DataSourceInfo getDataSourceInfo() {
		return this.dataInfo;
	}

	/**
	 * Sets the CWS Logger.
	 * 
	 * @param crawlerLogger
	 *            the CWS Logger.
	 */
	public final void setLogger(ICrawlerLog crawlerLogger) {
		this.logger = crawlerLogger;
	}

	/**
	 * Gets the CWS Logger. Note that the CWS logger is called infrequently.
	 * It's best to use the standard OpenLog logger.
	 * 
	 * @return the CWS Logger.
	 */
	public final ICrawlerLog getLogger() {
		return logger;
	}

	/**
	 * Gets the idc username.
	 * 
	 * @return stellent username that runs the crawl
	 */
	public String getIdcUsername() {
		return this.getDataSourceInfo().get(CrawlConstants.Provider.USERNAME);
	}

	/**
	 * Gets the idc url from the data source info.
	 * 
	 * @return idc url
	 */
	public String getIdcUrl() {
		return this.getDataSourceInfo().get(CrawlConstants.Provider.IDC_URL);
	}

	/**
	 * Pings the server. todo determine if we need to translate any error
	 * message.
	 * 
	 * @throws ServiceException
	 *             if pinging the server throws an IDC Client Exception.
	 */
	protected void pingServer() throws ServiceException {
		final String idcUrl = getIdcUrl();
		final String username = getIdcUsername();
		final IdcClientManager idcManager = new IdcClientManager();
		try {
			final IdcClient idcClient = idcManager.createClient(idcUrl);
			final IdcContext idcContext = new IdcContext(username);
			final DataBinder urlBinder = idcClient.createBinder();
			urlBinder.putLocal(UcmConstants.IDC_SERVICE, UcmConstants.Service.PING_SERVER);
			final ServiceResponse urlResponse = idcClient.sendRequest(idcContext, urlBinder);
			// we don't use this variable, but we want to see that the call
			// succeeds.
			final DataBinder urlResponseBinder = urlResponse.getResponseAsBinder();
		}
		catch (IdcClientException e) {
			LOGGER.error(e.getMessage(), e);
			// throw a service exception if the folder cannot be reached.
			throw new ServiceException(e);
		}
	}

}
