package com.oracle.wci.ucm;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Properties;
import java.util.TimeZone;

import javax.servlet.ServletContext;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Static Crawler Utilities.
 * 
 * @author <a href="mailto:cdavidso@bea.com">Clinton Davidson</a>
 * @version $Revision: #3 $ ($Change: 348620 $)
 */
public final class Utilities {

	/**
	 * Constants for the char encoding used in storing properties.
	 */
	private static final String ISO_8859_1 = "ISO-8859-1";
	/**
	 * Apache Commons Logger.
	 */
	private static final Log LOG = LogFactory.getLog(Utilities.class);

	/**
	 * Private class to prevent instantiation.
	 */
	private Utilities() {
		// no-op
	}

	/**
	 * Date format for folders in ucm.
	 */
	private static final String UCM_FOLDER_DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";

	/**
	 * Gets a date using the date format used in ucm. todo determine how to get
	 * the timezone todo determine what to do if the date is not parseable todo
	 * determine if ucm can use different date formats - if so, put the format
	 * into a properties file
	 * 
	 * @param dateString
	 *            date string from rdic response
	 * @return date
	 */
	public static Date getUcmDate(String dateString) {
		Date date = null;
		// if this ends with a z, trim it
		if (dateString.endsWith("Z")) {
			dateString = dateString.substring(0, dateString.length() - 1);
		}

		final DateFormat dateFormat = new SimpleDateFormat(UCM_FOLDER_DATE_FORMAT);
		final TimeZone timeZone = TimeZone.getDefault();
		dateFormat.setTimeZone(timeZone);
		try {
			date = dateFormat.parse(dateString);
		}
		catch (ParseException e) {
			e.printStackTrace();
		}
		return date;
	}

	/**
	 * Gets the temp directory, first trying the directory defined in the
	 * servlet context attribute "javax.servlet.context.tempdir" and then the
	 * System property "java.io.tmpdir".
	 * 
	 * @param context
	 *            ServletContext used to get the value of the attribute
	 *            "javax.servlet.context.tempdir".
	 * @return application server temp dir.
	 */
	public static File getTempDir(ServletContext context) {
		File tempDir = (File) context.getAttribute("javax.servlet.context.tempdir");

		if (null == tempDir || !tempDir.exists()) {
			final String tempDirLocation = System.getProperty("java.io.tmpdir");
			tempDir = new File(tempDirLocation);
		}
		return tempDir;
	}

	/**
	 * Trims a uuid of the curly braces so we can place it in a url and use it
	 * as a file name for indexing files.
	 * 
	 * @param UUID
	 *            uuid to trim
	 * @return uuid without opening and closing curly braces
	 */
	public static String getTrimmedUUID(String UUID) {
		String trimmedUUID = "";
		if (StringUtils.isNotBlank(UUID)) {
			trimmedUUID = UUID.substring(1, UUID.length() - 1);
		}
		return trimmedUUID;
	}

	/**
	 * Parses the locale string and returns a locale. The locale string may be
	 * null, or in the format "en" or "en_US".
	 * 
	 * @param localeString
	 *            string may be null, or in the format "en" or "en_US"
	 * @return Locale with language code and possibly country code, or
	 *         {@code Locale.ENGLISH} if no match was found.
	 */
	public static Locale parseLocaleString(String localeString) {
		Locale locale = null;
		if (null == localeString || localeString.length() < 2) {
			locale = Locale.getDefault();
		}
		else {
			final String lang = localeString.substring(0, 2);
			String country = "";
			if (localeString.length() > 4) {
				country = localeString.substring(3, 5);
			}
			locale = new Locale(lang, country);
			if (null == locale) {
				locale = Locale.ENGLISH;
			}
		}
		return locale;
	}

	/**
	 * Turns properties into a string to be used for
	 * {@link com.plumtree.remote.crawler.CrawlerConstants#TAG_PATH}. Uses
	 * ISO-8859-1 as that is the encoding used by Java for Properties
	 * 
	 * @param props
	 *            Properties
	 * @return String to be used for PTC_UNIQUE, i.e.
	 *         {@linkplain com.plumtree.remote.crawler.CrawlerConstants#TAG_PATH}
	 */
	public static String propsToString(Properties props) {
		try {
			final ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
			props.store(byteStream, null);
			// use the iso 8859 encoding as this is what is written to when
			// storing
			// todo see if this causes issues
			final String propStringWithComment = byteStream.toString(ISO_8859_1);
			// trim off the first comment line, as this includes a date which
			// causes the crawl
			// indexer to create duplicate documents
			final ByteArrayInputStream inputStream = new ByteArrayInputStream(propStringWithComment
					.getBytes(ISO_8859_1));
			final BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream,
					ISO_8859_1));
			final StringBuilder builder = new StringBuilder();
			// read the first line from the reader and ignore it
			reader.readLine();
			String line = reader.readLine();
			while (line != null) {
				builder.append(line);
				builder.append('\n');
				line = reader.readLine();
			}
			return builder.toString();
		}
		catch (IOException e) {
			LOG.error(e.getMessage(), e);
			return null;
		}
	}

	/**
	 * Deserializes the containerLocation or documentLocation string used in
	 * {@link com.plumtree.remote.crawler.IContainerProvider#attachToContainer(String, com.plumtree.remote.crawler.ChildRequestHint, String)}
	 * or
	 * {@link com.plumtree.remote.crawler.IDocumentProvider#attachToDocument(String, String, String, com.plumtree.remote.crawler.DocumentFormat)}
	 * into properties. If there is an IOException, logs it but does not
	 * propagate it.
	 * 
	 * @param input
	 *            location string serialized properties.
	 * @return Properties
	 */
	public static Properties stringToProperties(String input) {
		Properties props = null;
		try {
			final byte[] bytes = input.getBytes(ISO_8859_1);
			final ByteArrayInputStream byteStream = new ByteArrayInputStream(bytes);
			props = new Properties();
			props.load(byteStream);
			return props;
		}
		catch (IOException e) {
			LOG.error(e.getMessage(), e);
			return null;
		}
	}

}