/*
 * Copyright (c) 2008 BEA Systems, Inc. All Rights Reserved.
 *
 * $Id: //Dev/ucmcrawler/main/src/com/oracle/wci/ucm/container/ContainerProvider.java#4 $
 *
 * This software is the confidential and proprietary information of BEA Systems, Inc. ("Confidential
 * Information"). You shall not disclose such Confidential Information and shall use it only in accordance
 * with the terms of the license agreement you entered into with BEA Systems Inc.
 */

package com.oracle.wci.ucm.container;

import java.util.Locale;
import java.util.Properties;

import oracle.stellent.ridc.IdcClient;
import oracle.stellent.ridc.IdcClientException;
import oracle.stellent.ridc.IdcClientManager;
import oracle.stellent.ridc.IdcContext;
import oracle.stellent.ridc.model.DataBinder;
import oracle.stellent.ridc.protocol.ServiceResponse;

import com.oracle.wci.ucm.Provider;
import com.oracle.wci.ucm.CrawlConstants;
import com.oracle.wci.ucm.UcmConstants;
import com.oracle.wci.ucm.Utilities;
import com.plumtree.remote.ServiceException;
import com.plumtree.remote.crawler.ChildRequestHint;
import com.plumtree.remote.crawler.CrawlerInfo;
import com.plumtree.remote.crawler.IContainer;
import com.plumtree.remote.crawler.IContainerProvider;
import com.plumtree.remote.crawler.ICrawlerLog;
import com.plumtree.remote.util.DataSourceInfo;

/**
 * UCM implementation of IContainerProvider.
 * @author <a href="mailto:clinton.davidson@oracle.com">Clinton Davidson</a>
 * @version $Revision: #4 $ ($Change: 348620 $)
 */
public final class ContainerProvider extends Provider implements IContainerProvider {

	/**
	 * For an initial crawl, the location will be PTC_UNIQUE.
	 * 
	 * @param containerLocation
	 *            string which contains a properties file with the ucm id of the
	 *            container
	 * @param hint
	 *            ignored
	 * @param localeCode
	 *            locale of the crawl
	 * @return Initialized FolderContainer
	 * @throws ServiceException
	 *             if an exception occurred when retreiving the container
	 * @see com.plumtree.remote.crawler.IContainerProvider#attachToContainer(java.lang.String,
	 *      com.plumtree.remote.crawler.ChildRequestHint, java.lang.String)
	 */
	public IContainer attachToContainer(String containerLocation, ChildRequestHint hint,
			String localeCode) throws ServiceException {
		// the crawler location will be serialized properties- deserialize them
		// parse the containerLocation into properties and get the locale.
		final Properties properties = Utilities.stringToProperties(containerLocation);
		final Locale locale = Utilities.parseLocaleString(localeCode);

		// the connection information will be in the data source information,
		// not in the container location
		final String idcUrl = this.getIdcUrl();
		final String username = this.getIdcUsername();
		final String folderDID = properties.getProperty(CrawlConstants.Provider.START_FOLDER_ID);
		final IdcClientManager idcManager = new IdcClientManager();
		try {
			final IdcClient idcClient = idcManager.createClient(idcUrl);
			final IdcContext idcContext = new IdcContext(username);

			// The IdcClient is created and configured in the base class

			// Create the DataBinder and populated it with needed information
			// for
			// the service call
			final DataBinder binder = idcClient.createBinder();
			// get the list of documents in the folder, put into a hashmap where
			// the
			// key is the original name,and the value the data object
			binder.putLocal(UcmConstants.IDC_SERVICE, UcmConstants.Service.COLLECTION_DISPLAY);
			binder.putLocal(UcmConstants.Field.D_COLLECTION_ID, folderDID);
			binder.putLocal(UcmConstants.Field.HAS_COLLECTION_ID, String.valueOf(1));
			final ServiceResponse response = idcClient.sendRequest(idcContext, binder);
			final DataBinder resultsBinder = response.getResponseAsBinder();
			// construct the folder with the properties and the data binder.
			return new FolderContainer(properties, locale, this, resultsBinder);

		}
		catch (IdcClientException e) {
			// throw a service exception if the folder cannot be reached.
			throw new ServiceException(e);
		}

	}

	/**
	 * Stores the datasourceInfo, crawlerInfo, and crawlerLog.
	 * 
	 * @param dataSourceInfo
	 *            information from the data source, including the idc username
	 *            and url
	 * @param crawlerInfo
	 *            information from the crawler sci pages
	 * @param crawlerLog
	 *            crawler log. Keep in mind that this log is only called
	 *            erratically during the crawl.
	 * @throws ServiceException
	 *             if unable to ping the server
	 * @see com.plumtree.remote.crawler.IContainerProvider#initialize(com.plumtree.remote.util.DataSourceInfo,
	 *      com.plumtree.remote.crawler.CrawlerInfo,
	 *      com.plumtree.remote.crawler.ICrawlerLog)
	 */
	public void initialize(DataSourceInfo dataSourceInfo, CrawlerInfo crawlerInfo,
			ICrawlerLog crawlerLog) throws ServiceException {
		this.setDataSourceInfo(dataSourceInfo);
		this.setCrawlerInfo(crawlerInfo);
		this.setLogger(crawlerLog);
		pingServer();

	}

	/**
	 * Clears the security group map.
	 * 
	 * @throws ServiceException
	 *             never
	 * @see com.plumtree.remote.crawler.IContainerProvider#shutdown()
	 */
	public void shutdown() throws ServiceException {
		clearSecurityGroupRoleMap();
	}

}
