/*
 * Copyright (c) 2008 BEA Systems, Inc. All Rights Reserved.
 *
 * $Id: //Dev/ucmcrawler/main/src/com/oracle/wci/ucm/container/FolderContainer.java#4 $
 *
 * This software is the confidential and proprietary information of BEA Systems, Inc. ("Confidential
 * Information"). You shall not disclose such Confidential Information and shall use it only in accordance
 * with the terms of the license agreement you entered into with BEA Systems Inc.
 */

package com.oracle.wci.ucm.container;

import java.util.Locale;
import java.util.Properties;
import java.util.Set;

import oracle.stellent.ridc.model.DataBinder;
import oracle.stellent.ridc.model.DataObject;
import oracle.stellent.ridc.model.DataResultSet;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.oracle.wci.ucm.CrawlConstants;
import com.oracle.wci.ucm.Provider;
import com.oracle.wci.ucm.UcmConstants;
import com.oracle.wci.ucm.Utilities;
import com.plumtree.remote.ServiceException;
import com.plumtree.remote.crawler.ACLEntry;
import com.plumtree.remote.crawler.ChildContainer;
import com.plumtree.remote.crawler.ChildDocument;
import com.plumtree.remote.crawler.ContainerMetaData;
import com.plumtree.remote.crawler.IContainer;
import com.plumtree.remote.crawler.TypeNamespace;

/**
 * UCM implementation of IContainer.
 * @author <a href="mailto:clinton.davidson@oracle.com">Clinton Davidson</a>
 * @version $Revision: #4 $ ($Change: 348620 $)
 */
public final class FolderContainer implements IContainer {

	/**
	 * Apache Commons Logger.
	 */
	private static final Log LOGGER = LogFactory.getLog(FolderContainer.class);

	/**
	 * Provider, which includes methods to get ACLs.
	 */
	private final Provider containerProvider;

	/**
	 * Crawler locale. This is specified in the portal when the administrator
	 * selects "the majority of documents are in this language".
	 */
	private final Locale crawlerLocale;

	/**
	 * Gets the crawler locale.
	 * 
	 * @return the crawler locale.
	 */
	protected Locale getLocale() {
		return crawlerLocale;
	}

	/**
	 * Container properties, including space type, container type, space
	 * identifier and possibly page or record identifier.
	 */
	private final Properties containerProperties;

	/**
	 * Gets the container properties, which includes space type, container type,
	 * space identifier and possibly page or record identifier.
	 * 
	 * @return container properties
	 */
	private Properties getProperties() {
		return containerProperties;
	}

	/**
	 * Results binder with container information, and result sets for folders
	 * and files.
	 */
	private DataBinder resultsBinder;

	/**
	 * Default constructor.
	 * 
	 * @param properties
	 *            container properties
	 * @param locale
	 *            crawler locale
	 * @param provider
	 *            Provider
	 * @param resultsBinder
	 *            resultsBinder, which contains results sets for child documents
	 *            and child folders.
	 * 
	 * 
	 */
	public FolderContainer(Properties properties, Locale locale, Provider provider,
			DataBinder resultsBinder) {
		this.containerProperties = properties;
		this.crawlerLocale = locale;
		this.containerProvider = provider;
		this.resultsBinder = resultsBinder;
	}

	/**
	 * Gets subfolders of the current folder.
	 * 
	 * @return array of childcontainers. Keep in mine that this may not be
	 *         called if the crawl depth is limited.
	 * @throws ServiceException
	 *             never
	 * @see com.plumtree.remote.crawler.IContainer#getChildContainers()
	 */
	public ChildContainer[] getChildContainers() throws ServiceException {
		// get the result set of folders
		final DataResultSet resultSet = resultsBinder
				.getResultSet(UcmConstants.ResultSet.COLLECTIONS);
		final ChildContainer[] childContainers = new ChildContainer[resultSet.getRows().size()];
		int i = 0;
		for (DataObject dataObject : resultSet.getRows()) {
			final String ucmFolderName = dataObject.get(UcmConstants.Field.D_COLLECTION_NAME);
			final String ucmFolderId = dataObject.get(UcmConstants.Field.D_COLLECTION_ID);
			final Properties childProperties = new Properties();
			childProperties.putAll(containerProperties);
			// overwrite the start folder
			childProperties.setProperty(CrawlConstants.Provider.START_FOLDER_ID, ucmFolderId);
			final String location = Utilities.propsToString(childProperties);
			final ChildContainer childContainer = new ChildContainer(location, ucmFolderName);
			childContainers[i] = childContainer;
			i++;
		}
		return childContainers;
	}

	/**
	 * Gets child documents of the current folder.
	 * @return child documents of the current folder.
	 * @throws ServiceException
	 *             never
	 * @see com.plumtree.remote.crawler.IContainer#getChildDocuments()
	 */
	public ChildDocument[] getChildDocuments() throws ServiceException {
		final DataResultSet resultSet = resultsBinder.getResultSet(UcmConstants.ResultSet.CONTENTS);
		final ChildDocument[] childDocuments = new ChildDocument[resultSet.getRows().size()];
		int i = 0;
		for (DataObject dataObject : resultSet.getRows()) {
			final String ucmDocName = dataObject.get(UcmConstants.Field.D_ORIGINAL_NAME);
			final String ucmId = dataObject.get(UcmConstants.Field.D_ID);
			final Properties childProperties = new Properties();
			childProperties.putAll(containerProperties);
			// document id
			childProperties.setProperty(UcmConstants.Field.D_ID, ucmId);
			final String location = Utilities.propsToString(childProperties);
			final ChildDocument childDocument = new ChildDocument(location, TypeNamespace.FILE,
					ucmDocName, ucmDocName);
			childDocuments[i] = childDocument;
			i++;
		}
		return childDocuments;
	}

	/**
	 * Groups come from two sources: (a) the roles in the security group, which
	 * we assume map to LDAP or AD groups; (b) Accounts in the dDocAccount
	 * field, which we also assume to map to LDAP or AD groups.
	 * 
	 * @return Groups with access to this folder.
	 * @throws ServiceException
	 *             never
	 * @see com.plumtree.remote.crawler.IContainer#getGroups()
	 */
	public ACLEntry[] getGroups() throws ServiceException {
		final String securityGroupName = resultsBinder.getLocal(UcmConstants.Field.D_SECURITY_GROUP);

		// get the groups from the provider- this should always exist
		final Set<ACLEntry> roles = containerProvider.getRolesFromSecurityGroup(securityGroupName);
		// add the account, if it is not empty
		final String account = resultsBinder.getLocal(UcmConstants.Field.D_DOC_ACCOUNT);
		if (StringUtils.isNotEmpty(account)) {
			final ACLEntry entry = new ACLEntry();
			entry.setDomain(CrawlConstants.Misc.DOMAIN_NAME);
			entry.setName(account);
			roles.add(entry);
		}
		return roles.toArray(new ACLEntry[roles.size()]);

	}

	/**
	 * Returns an empty ContainerMetaData as this is ignored by the portal.
	 * 
	 * @param arg0
	 *            ignored
	 * @return blank ContainerMetaData
	 * @throws ServiceException
	 *             never, as a blank ContainerMetaData is always returned
	 * @see com.plumtree.remote.crawler.IContainer#getMetaData(java.lang.String[])
	 */
	@Deprecated
	public ContainerMetaData getMetaData(String[] arg0) throws ServiceException {
		return new ContainerMetaData();
	}

	/**
	 * Returns a 0-len array, as we are not importing users.
	 * 
	 * @return users defined in all roles for this space.
	 * @throws ServiceException
	 *             if an error occurred when retrieving groups
	 * @see com.plumtree.remote.crawler.IContainer#getUsers()
	 */
	public ACLEntry[] getUsers() throws ServiceException {
		return new ACLEntry[0];
	}

}
