/*
 * Copyright (c) 2008 BEA Systems, Inc. All Rights Reserved.
 *
 * $Id: //Dev/ucmcrawler/main/src/com/oracle/wci/ucm/document/DocumentProvider.java#6 $
 *
 * This software is the confidential and proprietary information of BEA Systems, Inc. ("Confidential
 * Information"). You shall not disclose such Confidential Information and shall use it only in accordance
 * with the terms of the license agreement you entered into with BEA Systems Inc.
 */

package com.oracle.wci.ucm.document;

import java.io.File;
import java.io.FileFilter;
import java.util.Locale;
import java.util.Properties;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServlet;

import oracle.stellent.ridc.IdcClient;
import oracle.stellent.ridc.IdcClientException;
import oracle.stellent.ridc.IdcClientManager;
import oracle.stellent.ridc.IdcContext;
import oracle.stellent.ridc.model.DataBinder;
import oracle.stellent.ridc.protocol.ServiceResponse;

import org.apache.axis.MessageContext;
import org.apache.axis.transport.http.HTTPConstants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.oracle.wci.ucm.Provider;
import com.oracle.wci.ucm.UcmConstants;
import com.oracle.wci.ucm.Utilities;
import com.plumtree.remote.NoLongerExistsException;
import com.plumtree.remote.ServiceException;
import com.plumtree.remote.crawler.DocumentFormat;
import com.plumtree.remote.crawler.ICrawlerLog;
import com.plumtree.remote.crawler.IDocument;
import com.plumtree.remote.crawler.IDocumentProvider;
import com.plumtree.remote.util.DataSourceInfo;

/**
 * UCM implementation of IDocumentProvider.
 * @author <a href="mailto:clinton.davidson@oracle.com">Clinton Davidson</a>
 * @version $Revision: #6 $ ($Change: 348620 $)
 */
public final class DocumentProvider extends Provider implements IDocumentProvider {
	/**
	 * Status code for no longer exists.
	 */
	private static final int STATUS_CODE_DOES_NOT_EXIST = -16;
	/**
	 * Apache Commons Logger.
	 */
	public static final Log LOGGER = LogFactory.getLog(DocumentProvider.class);

	/**
	 * Connects to the document specified by the dID in the documentLocation.
	 * 
	 * @param documentLocation
	 *            string which contains the document dID
	 * @param localeString
	 *            used to determine the document locale
	 * @param signature
	 *            existing document signature
	 * @param format
	 *            ignored
	 * @return Initialized FileDocument
	 * @throws ServiceException
	 *             if unable to get the document. This could be a subclass of
	 *             ServiceException showing a permission exception, or showing
	 *             that the file has been deleted and should be removed from the
	 *             portal.
	 * @see com.plumtree.remote.crawler.IDocumentProvider#attachToDocument(java.lang.String,
	 *      java.lang.String, java.lang.String,
	 *      com.plumtree.remote.crawler.DocumentFormat)
	 */
	public IDocument attachToDocument(String documentLocation, String localeString,
			String signature, DocumentFormat format) throws ServiceException {
		final Properties properties = Utilities.stringToProperties(documentLocation);
		final Locale locale = Utilities.parseLocaleString(localeString);
		// connect to ucm to get the document information
		final String idcUrl = getIdcUrl();
		final String username = getIdcUsername();
		final String dID = properties.getProperty(UcmConstants.Field.D_ID);
		final IdcClientManager idcManager = new IdcClientManager();
		try {
			final IdcClient idcClient = idcManager.createClient(idcUrl);
			final IdcContext idcContext = new IdcContext(username);
			final DataBinder urlBinder = idcClient.createBinder();
			urlBinder.putLocal(UcmConstants.IDC_SERVICE, UcmConstants.Service.DOC_INFO);
			urlBinder.putLocal(UcmConstants.Field.D_ID, dID);
			final ServiceResponse urlResponse = idcClient.sendRequest(idcContext, urlBinder);
			final DataBinder urlResponseBinder = urlResponse.getResponseAsBinder();
			final FileDocument document = new FileDocument(properties, locale, this,
					urlResponseBinder);
			return document;
		}

		catch (IdcClientException e) {
			final String message = e.getMessage();
			LOGGER.error(message, e);
			// we have to do message parsing to determine if this means that the
			// item does not exist
			// hopefully this message is not localized
			// note that the document still exists if it is in the trash.
			if (e instanceof oracle.stellent.ridc.protocol.ServiceException) {
				final oracle.stellent.ridc.protocol.ServiceException se = (oracle.stellent.ridc.protocol.ServiceException) e;
				// status code of -16 means no longer exists
				if (se.getStatusCode() == STATUS_CODE_DOES_NOT_EXIST) {
					throw new NoLongerExistsException(message);
				}
			}

			throw new ServiceException(e);
		}

	}

	/**
	 * Sets the data source info and the crawler logger. Pings the server, and
	 * returns a service exception if the server is not up.
	 * 
	 * @param dataSourceInfo
	 *            connection information
	 * @param crawlerLogger
	 *            logger which is called intermittantly by the portal
	 * @throws ServiceException
	 *             if unable to connect to UCM.
	 * @see com.plumtree.remote.crawler.IDocumentProvider#initialize(com.plumtree.remote.util.DataSourceInfo,
	 *      com.plumtree.remote.crawler.ICrawlerLog)
	 */
	public void initialize(DataSourceInfo dataSourceInfo, ICrawlerLog crawlerLogger)
			throws ServiceException {
		this.setDataSourceInfo(dataSourceInfo);
		this.setLogger(crawlerLogger);
		pingServer();

	}

	/**
	 * Shutdown is unreliable, but if this method is called, it will clear the
	 * security group map and delete any files ending with ".tmp" in the
	 * application server temp dir. These files should have deleted by the index
	 * servlet, but this will delete any remaining files.
	 * 
	 * @throws ServiceException
	 *             ignored, as a thrown exception at this point will have no
	 *             effect on the crawl.
	 * @see com.plumtree.remote.crawler.IDocumentProvider#shutdown()
	 */
	public void shutdown() throws ServiceException {
		clearSecurityGroupRoleMap();
		final MessageContext messageContext = MessageContext.getCurrentContext();
		final HttpServlet servlet = (HttpServlet) messageContext
				.getProperty(HTTPConstants.MC_HTTP_SERVLET);
		final ServletContext context = servlet.getServletContext();
		final File indexLocation = Utilities.getTempDir(context);
		final File[] tempFiles = indexLocation.listFiles(new TempFileFilter());
		for (int i = 0; i < tempFiles.length; i++) {
			final File tempFile = tempFiles[i];
			if (tempFile.exists()) {
				tempFile.delete();
			}
		}
		LOGGER.info("Document Provider shutting down");

	}

	/**
	 * FileFilter for files generated for the indexer servlet.
	 * 
	 * @author <a href="mailto:cdavidso@bea.com">Clinton Davidson</a>
	 */
	private static class TempFileFilter implements FileFilter {

		/**
		 * Extension for generated temp indexing files.
		 */
		private static final String TEMP_FILE_EXTENSION = ".tmp";

		/**
		 * Accepts files that end in "&#46;tmp".
		 * 
		 * @param pathname
		 *            pathname to check if it ends in "&#46;tmp"
		 * @return {@code true} if the file ends with &#46;tmp
		 * @see java.io.FileFilter#accept(java.io.File)
		 */
		public boolean accept(File pathname) {
			boolean acceptFile = false;
			if (null != pathname) {
				final String fileName = pathname.getName();
				if (fileName.endsWith(TEMP_FILE_EXTENSION)) {
					acceptFile = true;
				}
			}
			return acceptFile;
		}
	}

}
