/*
 * Copyright (c) 2008 BEA Systems, Inc. All Rights Reserved.
 *
 * $Id: //Dev/ucmcrawler/main/src/com/oracle/wci/ucm/document/FileDocument.java#4 $
 *
 * This software is the confidential and proprietary information of BEA Systems, Inc. ("Confidential
 * Information"). You shall not disclose such Confidential Information and shall use it only in accordance
 * with the terms of the license agreement you entered into with BEA Systems Inc.
 */

package com.oracle.wci.ucm.document;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.Set;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;

import oracle.stellent.ridc.IdcClient;
import oracle.stellent.ridc.IdcClientManager;
import oracle.stellent.ridc.IdcContext;
import oracle.stellent.ridc.model.DataBinder;
import oracle.stellent.ridc.model.DataObject;
import oracle.stellent.ridc.model.DataResultSet;
import oracle.stellent.ridc.protocol.ServiceResponse;

import org.apache.axis.MessageContext;
import org.apache.axis.transport.http.HTTPConstants;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.FastDateFormat;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.oracle.wci.ucm.CrawlConstants;
import com.oracle.wci.ucm.Provider;
import com.oracle.wci.ucm.UcmConstants;
import com.oracle.wci.ucm.Utilities;
import com.plumtree.remote.ServiceException;
import com.plumtree.remote.crawler.ACLEntry;
import com.plumtree.remote.crawler.DocumentMetaData;
import com.plumtree.remote.crawler.IDocument;

/**
 * UCM implementation of IDocument.
 * @author <a href="mailto:clinton.davidson@oracle.com">Clinton Davidson</a>
 * @version $Revision: #4 $ ($Change: 348620 $)
 */
public final class FileDocument implements IDocument {
/**
 * Properties file of ucmFieldNames.properties which includes optional ucm field names to crawl.
 */
	private static Properties ucmFieldNames;
	

	/**
	 * Apache Commons Logger.
	 */
	private static final Log LOGGER = LogFactory.getLog(FileDocument.class);


	/**
	 * Fast date format for tasks. Date formats are in UTC, so set the time zone
	 * to utc.
	 */
	private static final FastDateFormat PORTAL_DATE_FORMAT = FastDateFormat.getDateTimeInstance(
			DateFormat.SHORT, DateFormat.SHORT);

	// todo put this in a listener for better error reporting
	static {
		try {
			final InputStream inputStream = FileDocument.class
					.getResourceAsStream("/ucmFieldNames.properties");
			ucmFieldNames = new Properties();
			ucmFieldNames.load(inputStream);
		}
		catch (IOException e) {
			LOGGER.error(e.getMessage(), e);
		}
	}

	/**
	 * Provider, which provides access to data source info and ACLs.
	 */
	private final Provider documentProvider;

	/**
	 * Base index location, which is where temp files will be created.
	 */
	private static File baseIndexLocation;

	/**
	 * Buffer size for writing from document to temp file whose value is
	 * {@value}.
	 */
	private static final int BUFFER_SIZE = (40 * 1024);

	/**
	 * Base url of the indexing servlet.
	 */
	private static String baseIndexURL;

	/**
	 * Crawler locale. This is specified in the portal when the administrator
	 * selects "the majority of documents are in this language".
	 */
	private final Locale locale;

	/**
	 * Document properties.
	 */
	private final Properties documentProperties;

	/**
	 * Gets the crawler locale.
	 * 
	 * @return the crawler locale.
	 */
	protected Locale getLocale() {
		return locale;
	}

	/**
	 * Maximum number of chars in metadata; do not set this longer than 255 or
	 * the crawler job will bomb.
	 */
	private static final int MAX_CHARS_METADATA = 255;
	/**
	 * DataBinder for accessing local data, such as the document url.
	 */
	private final DataBinder responseBinder;
	/**
	 * DataObject with contains the row from the DOC_INFO result set.
	 */
	private final DataObject dataObject;

	/**
	 * Default Constructor.
	 * 
	 * @param properties
	 *            child document properties
	 * @param crawlerLocale
	 *            crawler locale
	 * @param provider
	 *            Provider, which provides access to data source info and ACLs
	 * @param responseBinder
	 *            DataBinder with local field information, as well as a DOC_INFO
	 *            result set.
	 */
	public FileDocument(Properties properties, Locale crawlerLocale, Provider provider,
			DataBinder responseBinder) {
		this.documentProperties = properties;
		this.locale = crawlerLocale;
		this.documentProvider = provider;
		this.responseBinder = responseBinder;
		// set the data object
		final DataResultSet docInfoRevisionResultSet = responseBinder
				.getResultSet(UcmConstants.ResultSet.DOC_INFO);
		final List<DataObject> docInfoRevisionRows = docInfoRevisionResultSet.getRows();
		final DataObject docInfoRevisionRow = docInfoRevisionRows.get(0);
		this.dataObject = docInfoRevisionRow;
	}

	/**
	 * Not using docFetch, so return "".
	 * 
	 * @return empty string
	 * @throws ServiceException
	 *             ignored, as we are not retrieving a file
	 * @see com.plumtree.remote.crawler.IDocument#getDocument()
	 */
	public String getDocument() throws ServiceException {
		return "";
	}

	/**
	 * Gets the ucm last modified date. We do not need to worry about the
	 * formatting, as the portal just checks if the string is the same. todo:
	 * determine if release date is the proper field.
	 * @return ucm last modified date
	 * @throws ServiceException never
	 * @see com.plumtree.remote.crawler.IDocument#getDocumentSignature()
	 */
	public String getDocumentSignature() throws ServiceException {
		return dataObject.get(UcmConstants.Field.D_RELEASE_DATE);
	}

	/**
	 * Gets security groups for a document, which includes roles in the security group for the document, and anything listed in accounts.
	 * 
	 * @return array of ACLEntry, where the domain is CrawlConstants#Misc#DOMAIN_NAME
	 * @throws ServiceException required by api
	 * @see com.plumtree.remote.crawler.IDocument#getGroups()
	 */
	public ACLEntry[] getGroups() throws ServiceException {
		final String securityGroupName = dataObject.get(UcmConstants.Field.D_SECURITY_GROUP);

		// get the groups from the provider- this should always exist
		final Set<ACLEntry> roles = documentProvider.getRolesFromSecurityGroup(securityGroupName);
		// add the account, if it is not empty
		final String account = dataObject.get(UcmConstants.Field.D_DOC_ACCOUNT);
		if (StringUtils.isNotEmpty(account)) {
			final ACLEntry entry = new ACLEntry();
			entry.setDomain(CrawlConstants.Misc.DOMAIN_NAME);
			entry.setName(account);
			roles.add(entry);
		}
		return roles.toArray(new ACLEntry[roles.size()]);
	}

	/**
	 * Adds the required metadata, as well as any fields listed in the
	 * properties file ucmFieldNames.properties. Only fields with non-blank
	 * values are added to metadata. Metadata will only be added to a crawled
	 * document if the property exists in the Global Document Property Map.
	 * @param arg0 ignored
	 * @return document metadata, including any values specified in the properties file ucmFieldNames.properties
	 * @throws ServiceException never
	 * @see com.plumtree.remote.crawler.IDocument#getMetaData(java.lang.String[])
	 */
	public DocumentMetaData getMetaData(String[] arg0) throws ServiceException {
		final DocumentMetaData metaData = new DocumentMetaData();
		metaData.setUseDocFetch(false);
		final String docFormats = responseBinder.getLocal(UcmConstants.Field.D_DOC_FORMATS);
		metaData.setContentType(docFormats);

		final String title = dataObject.get(UcmConstants.Field.D_DOC_TITLE);
		metaData.setName(title);

		final String fileName = dataObject.get(UcmConstants.Field.D_ORIGINAL_NAME);
		metaData.setFileName(fileName);

		String description = dataObject.get(UcmConstants.Field.X_COMMENTS);
		// check that comments are not blank
		if (StringUtils.isEmpty(description)) {
			description = "";
		}
		// check that the description is not more than 255 chars
		description = StringUtils.abbreviate(description, FileDocument.MAX_CHARS_METADATA);
		metaData.setDescription(description);
		final String dId = dataObject.get(UcmConstants.Field.D_ID);
		metaData.put(UcmConstants.Field.D_ID, dId);

		// add the index url
		final String indexUrl = this.getIndexURL(metaData);
		metaData.setIndexingURL(indexUrl);

		// set the click through url to be the stellent url
		final String url = responseBinder.getLocal(UcmConstants.Field.DOC_URL);
		metaData.setClickThroughURL(url);

		// metadata from local data
		final String[] localFieldNames = getTrimmedFieldNames("localFieldNames");
		addStringMetaDataFromBinder(localFieldNames, metaData);

		// metadata from dataobject
		final String[] dataObjectFieldNames = getTrimmedFieldNames("dataObjectStringFieldNames");
		addStringMetaDataFromDataObject(dataObjectFieldNames, metaData);

		// date metadata from dataobject
		final String[] dataObjectDateFieldNames = getTrimmedFieldNames("dataObjectDateFieldNames");
		addDateMetaDataFromDataObject(dataObjectDateFieldNames, metaData);
		// no ints at this point
		return metaData;
	}

	/**
	 * Gets trimmed field names from the corresponding property from the file
	 * ucmFieldNames.properties.
	 * 
	 * @param propertyName
	 *            name of the property in ucmFieldNames.properties
	 * @return String[] of trimmed stellent field names.
	 */
	private String[] getTrimmedFieldNames(String propertyName) {
		String[] splitFieldNames = new String[0];
		final String unSplit = ucmFieldNames.getProperty(propertyName);
		if (StringUtils.isNotBlank(unSplit)) {
			final String[] untrimmedFieldNames = unSplit.split(",");
			final int length = untrimmedFieldNames.length;
			splitFieldNames = new String[length];
			for (int i = 0; i < length; i++) {
				splitFieldNames[i] = untrimmedFieldNames[i].trim();
			}
		}
		return splitFieldNames;
	}

	/**
	 * Adds string metadata from the DOC_INFO result set to document metadata.
	 * 
	 * @param fieldNames
	 *            array of field names to add. Only fields with non-blank values
	 *            will be added.
	 * @param metaData
	 *            metadata to add to
	 */
	private void addStringMetaDataFromDataObject(String[] fieldNames, DocumentMetaData metaData) {
		for (String fieldName : fieldNames) {
			// get the value from the binder
			final String value = dataObject.get(fieldName);
			addCheckedFieldToMetaData(metaData, fieldName, value);
		}
	}

	/**
	 * Adds date metadata from the DOC_INFO result set to document metadata. The
	 * field names are assumed to be data types, as the value is parsed into a
	 * date, and then formatted into the portal format.
	 * 
	 * @param fieldNames
	 *            array of field names to add. Only fields with non-blank values
	 *            will be added.
	 * @param metaData
	 *            metadata to add to
	 */
	private void addDateMetaDataFromDataObject(String[] fieldNames, DocumentMetaData metaData) {
		for (String fieldName : fieldNames) {
			// get the value from the binder
			final String value = dataObject.get(fieldName);
			if (StringUtils.isNotBlank(value)) {
				final String portalDateString = this.getPortalDateString(value);
				metaData.put(fieldName, portalDateString);
			}
		}
	}

	/**
	 * Adds metadata from the local data of the DataBinder to document metadata.
	 * 
	 * @param fieldNames
	 *            array of field names to add. Only fields with non-blank values
	 *            will be added.
	 * @param metaData
	 *            metadata to add to
	 */
	private void addStringMetaDataFromBinder(String[] fieldNames, DocumentMetaData metaData) {
		for (String fieldName : fieldNames) {
			// get the value from the binder
			final String value = responseBinder.getLocal(fieldName);
			addCheckedFieldToMetaData(metaData, fieldName, value);
		}
	}

	/**
	 * Adds a stellent field value to the document metadata. Only non-blank
	 * values are added; values that are greater than 255 characters are trimmed
	 * to 255, as the portal bombs on metadata greater than this limit.
	 * 
	 * @param metaData
	 *            document metadata
	 * @param fieldName
	 *            stellent field name
	 * @param value
	 *            stellend field value
	 */
	private void addCheckedFieldToMetaData(DocumentMetaData metaData, String fieldName, String value) {
		// check that it is not blank
		if (StringUtils.isNotBlank(value)) {
			// check that it is not more than 255 chars
			final String abbreviatedValue = StringUtils.abbreviate(value, FileDocument.MAX_CHARS_METADATA);

			// set in the doc metadata
			metaData.put(fieldName, abbreviatedValue);
		}
	}

	/**
	 * Returns an empty array, as getting security by user is not supported.
	 * @return 0-len array
	 * @throws ServiceException never
	 * @see com.plumtree.remote.crawler.IDocument#getUsers()
	 */
	public ACLEntry[] getUsers() throws ServiceException {
		return new ACLEntry[0];
	}

	/**
	 * Gets the base index location, which is where temp files will be created.
	 * 
	 * @return the base index location, which is where temp files will be
	 *         created
	 */
	private File getBaseIndexLocation() {
		if (null == baseIndexLocation) {
			final MessageContext messageContext = MessageContext.getCurrentContext();
			final HttpServlet servlet = (HttpServlet) messageContext
					.getProperty(HTTPConstants.MC_HTTP_SERVLET);
			final ServletContext context = servlet.getServletContext();
			baseIndexLocation = Utilities.getTempDir(context);
		}
		return baseIndexLocation;
	}

	/**
	 * Gets the URL to the http context from the Axis MessageContext.
	 * 
	 * @return the URL to the http context
	 */
	private String getContextURL() {
		final MessageContext messageContext = MessageContext.getCurrentContext();
		final HttpServletRequest request = (HttpServletRequest) messageContext
				.getProperty(HTTPConstants.MC_HTTP_SERVLETREQUEST);
		final StringBuilder buffer = new StringBuilder();
		buffer.append(request.getScheme()).append("://").append(request.getServerName())
				.append(':').append(request.getServerPort()).append(request.getContextPath());
		return buffer.toString();
	}

	/**
	 * Gets the url for the index servlet.
	 * 
	 * @param metaData
	 *            DocumentMetaData metadata for indexing.
	 * @return url to the index servlet
	 */
	protected String getIndexURL(DocumentMetaData metaData) {

		final String dId = metaData.get(UcmConstants.Field.D_ID);
		final String tempFilePath = createTempFile(dId);
		writeIndexFile(tempFilePath, metaData);
		final String contentType = metaData.getContentType();
		String encodedContentType = contentType;
		try {
			encodedContentType = URLEncoder.encode(contentType, CrawlConstants.Misc.UTF8);
		}
		catch (Exception e) {
			LOGGER.error(e.getMessage(), e);
		}
		final String tempFileName = new File(tempFilePath).getName();
		// this does not need to be encoded, as it will always be a number +
		// ".tmp".
		final StringBuilder builder = new StringBuilder(getBaseIndexURL());
		builder.append('?').append(CrawlConstants.Misc.TEMP_FILENAME).append('=').append(
				tempFileName).append('&').append(CrawlConstants.Misc.CONTENT_TYPE).append('=')
				.append(encodedContentType);
		// todo add security by adding a token to the request
		// could also check for valid host in the servlet
		return builder.toString();
	}

	/**
	 * Writes out the temp indexing file.
	 * 
	 * @param tempFilePath
	 *            path to the temp file for writing the metaData or underlying
	 *            document
	 * @param metaData
	 *            DocumentMetaData metadata for indexing, which should include
	 *            the document id (dID)
	 */
	protected void writeIndexFile(String tempFilePath, DocumentMetaData metaData) {
		final File tempFile = new File(tempFilePath);
		// get the input stream from the stellent call
		final String idcUrl = documentProvider.getIdcUrl();
		final String username = documentProvider.getIdcUsername();
		final String dID = documentProperties.getProperty(UcmConstants.Field.D_ID);
		final IdcClientManager idcManager = new IdcClientManager();
		InputStream inputStream = null;
		FileOutputStream outputStream = null;
		try {
			final IdcClient idcClient = idcManager.createClient(idcUrl);
			final IdcContext idcContext = new IdcContext(username);

			final DataBinder binder = idcClient.createBinder();
			binder.putLocal(UcmConstants.IDC_SERVICE, UcmConstants.Service.GET_FILE);
			binder.putLocal(UcmConstants.Field.D_ID, dID);
			binder.putLocal(UcmConstants.Field.RENDITION, "Primary");

			final ServiceResponse response = idcClient.sendRequest(idcContext, binder);
			inputStream = response.getResponseStream();

			// stream out the file
			outputStream = new FileOutputStream(tempFile);
			final byte[] buf = new byte[BUFFER_SIZE];

			while (true) {
				final int bytesRead = inputStream.read(buf);
				if (bytesRead == -1) {
					break;
				}
				outputStream.write(buf, 0, bytesRead);
			}
		}
		catch (Exception e) {
			// this could be a document not found exception, or an IO exception
			LOGGER.error(e.getMessage(), e);
			documentProvider.getLogger().Log(e.getMessage());

		} finally {
			try {
				if (null != inputStream) {
					inputStream.close();
				}
				if (null != outputStream) {
					outputStream.close();
				}
			}
			catch (Exception e) {
				// log but do not propagate
				LOGGER.warn(e.getMessage(), e);
			}

		}
	}

	/**
	 * Gets the base url of the indexing servlet.
	 * 
	 * @return the base url of the indexing servlet
	 */
	protected String getBaseIndexURL() {
		// calculate once- should probably be static
		if (null == baseIndexURL) {
			baseIndexURL = getContextURL() + "/crawlindexer";
		}
		return baseIndexURL;
	}

	/**
	 * Creates a temp file and returns the absolute path of the newly created
	 * file.
	 * 
	 * @param dId dId of the ucm file- will be unique.
	 * @return absolute path of newly created temp file
	 */
	private String createTempFile(String dId) {
		File tempFile = null;
		try {
			tempFile = new File(getBaseIndexLocation(), dId + ".tmp");
			final FileWriter writer = new FileWriter(tempFile);
			writer.close();
		}
		catch (IOException io) {
			LOGGER.error(io.getMessage(), io);
		}
		return tempFile.getAbsolutePath();
	}

	/**
	 * Transforms a ucm date string in to the type of string that the portal
	 * expects.
	 * 
	 * @param stellentDateString
	 *            date string from ucm
	 * @return date string using the short date time format
	 */
	private String getPortalDateString(String stellentDateString) {
		// for dates, get the value in the format mm/dd/yyyy- the portal does
		// not use the time portion.
		// this could be done as a fast date format
		final Date date = Utilities.getUcmDate(stellentDateString);
		return PORTAL_DATE_FORMAT.format(date);
	}

}
