package com.oracle.wci.ucm.sci;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;
import com.oracle.wci.ucm.Utilities;

import com.plumtree.remote.ServiceException;
import com.plumtree.remote.sci.IAdminPage;
import com.plumtree.remote.util.NamedValue;
import com.plumtree.remote.util.NamedValueMap;

/**
 * Common methods for the {@link DatasourceEditor} and {@link CrawlerEditor}.
 * 
 * @author <a href="mailto:cdavidso@bea.com">Clinton Davidson</a>
 * @version $Revision: #3 $ ($Change: 348620 $)
 */
public abstract class AbstractEditor {


	/**
	 * Data source info for crawlers.
	 */
	private final NamedValueMap dataSourceInfo;
	/**
	 * Named value map of settings to be added-for example, password, type of
	 * space, and which space to add.
	 */
	private final NamedValueMap settings = new NamedValueMap();

	/**
	 * Locale used to display the appropriate language.
	 */
	private final Locale locale;

	/**
	 * Gets the resource bundle set for localized title, instructions, and field
	 * labels.
	 * 
	 */
	private final ResourceBundle resourceBundle;

	/**
	 * Default Constructor. Parses the locale, and looks up the
	 * ResourceBundleSet.
	 * 
	 * @param localeCode
	 *            locale in the format null : "en" : "en_US"
	 * @param parentInfo
	 *            NamedValueMap from the parent object, i.e. the data source.
	 */
	protected AbstractEditor(String localeCode, NamedValueMap parentInfo) {

		dataSourceInfo = parentInfo;
		// parse the locale code
		locale = Utilities.parseLocaleString(localeCode);
		// get the resource bundle.

		resourceBundle = ResourceBundle.getBundle("UcmCrawlerBundle", locale);
	}

	/**
	 * Gets the ResourceBundle for localized labels.
	 * 
	 * @return the ResourceBundle
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

	/**
	 * Implementation provided by the subclass of DatasourceEditor or
	 * CrawlerEditor.
	 * @return pages for the respective editor.
	 * @throws ServiceException if the content for the page could not be retrieved
	 * @see com.plumtree.remote.sci.IAdminEditor#getPages()
	 */
	public abstract IAdminPage[] getPages() throws ServiceException;

	/**
	 * Implementation provided by the subclass of DatasourceEditor or
	 * CrawlerEditor.
	 * @param pageInfo values passed into finalize
	 * @return array of named values - values may be added or subtracted from pageInfo
	 * @throws ServiceException if an exception occurred when finalizing
	 * @see com.plumtree.remote.sci.IAdminEditor#finalize(com.plumtree.remote.util.NamedValueMap)
	 */
	public abstract NamedValue[] finalize(NamedValueMap pageInfo)
	        throws ServiceException;

	/**
	 * Gets the crawler locale.
	 * 
	 * @return the crawler locale.
	 */
	final Locale getLocale() {
		return locale;
	}

	/**
	 * Gets the parent info, that is, the data source settings for the crawler.
	 * 
	 * @return the parent info.
	 */
	final NamedValueMap getParentInfo() {
		return dataSourceInfo;
	}

	/**
	 * Gets the settings, that is, the named values set in previous pages or in
	 * previous sessions.
	 * 
	 * @return the named values previously set
	 */
	final NamedValueMap getSettings() {
		return settings;
	}

	/**
	 * Adds additional settings to the named value map. Used when adding a
	 * setting that is not in the sci page itself.
	 * 
	 * @param name
	 *            name of the named value
	 * @param value
	 *            value of the named value
	 */
	final void addSettings(String name, String value) {
		settings.put(name, value);
	}
	/**
	 * Map of int validation codes to exception messages.
	 * A map is used as we can't know all exception messages in advance.
	 */
	private final Map<Integer, String> validationMessageMap = new HashMap<Integer, String>();
	
	/**
	 * Adds an error message for validation exceptions.
	 * @param errorCode validation error code
	 * @param exceptionMessage validation error message
	 */
	protected void addValidationExceptionMessage(int errorCode, String exceptionMessage){
		validationMessageMap.put(errorCode, exceptionMessage);
	}
	
	/**
	 * Gets the validation exception message matching the validation error code.
	 * @param errorCode validation error code, predefined as a constant in the SCI page.
	 * @return the validation exception message matching the validation error code
	 */
	protected String getValidationExceptionMessage(int errorCode){
		return validationMessageMap.get(errorCode);
	}

}
