package com.oracle.wci.ucm.sci;

import com.plumtree.remote.ServiceException;
import com.plumtree.remote.sci.IAdminPage;
import com.plumtree.remote.sci.ValidationException;
import com.plumtree.remote.util.NamedValueMap;

/**
 * Abstract superclass of all pages which provides an additional method to get
 * the editor. The methods {@link #getHelpURI()} and {@link #getImageURI()}
 * return "", as these are ignored in the 6.0 portal.
 * 
 * @author <a href="mailto:cdavidso@bea.com">Clinton Davidson</a>
 * @version $Revision: #3 $ ($Change: 348620 $)
 */
public abstract class AbstractPage implements IAdminPage {


	/**
	 * AbstractEditor, used to get the existing settings and add new settings.
	 */
	private final AbstractEditor abstractEditor;

	/**
	 * Default constructor, which sets the editor and locale.
	 * 
	 * @param editor
	 *            subclass of AbstractEditor, either a DatasourceEditor or
	 *            CrawlerEditor
	 */
	AbstractPage(AbstractEditor editor) {
		this.abstractEditor = editor;

	}

	/**
	 * Implemented by subclasses.
	 * @param errorCode error code thrown by validation exception, if any
	 * @param pageInfo named values from previous pages of the editor
	 * @return xml from sci elements as a string
	 * @throws ServiceException require by api
	 * @see com.plumtree.remote.sci.IAdminPage#getContent(int,
	 *      com.plumtree.remote.util.NamedValueMap)
	 */
	public abstract String getContent(int errorCode, NamedValueMap pageInfo)
	        throws ServiceException;

	/**
	 * Returns "", as help URI is not supported in the 6.x portal.
	 * @return empty string
	 * @throws ServiceException never
	 * @see com.plumtree.remote.sci.IAdminPage#getHelpURI()
	 */
	public String getHelpURI() throws ServiceException {
		return "";
	}

	/**
	 * Returns "", as image URI is not supported in the 6.x portal.
	 * @return empty string
	 *  @throws ServiceException never
	 * @see com.plumtree.remote.sci.IAdminPage#getImageURI()
	 */
	public String getImageURI() throws ServiceException {
		return "";
	}

	/**
	 * Gets the localized instructions in subclasses.
	 * @return the localized instructions in subclasses
	 *  @throws ServiceException never
	 * @see com.plumtree.remote.sci.IAdminPage#getInstructions()
	 */
	public abstract String getInstructions() throws ServiceException;

	/**
	 * Gets the localized title in subclasses.
	 * @return the localized title in subclasses
	 *  @throws ServiceException never
	 * @see com.plumtree.remote.sci.IAdminPage#getTitle()
	 */
	public abstract String getTitle() throws ServiceException;

	/**
	 * Validates the page in subclasses.
	 * @param pageInfo named-value pairs from page
	 * @throws ValidationException thrown by the developer if an error occurs in validation, the errorCode is then examined in getContent
	 * @throws ServiceException required by api
	 * @see com.plumtree.remote.sci.IAdminPage#validatePage(com.plumtree.remote.util.NamedValueMap)
	 */
	public abstract void validatePage(NamedValueMap pageInfo)
	        throws ValidationException, ServiceException;

	/**
	 * Gets the editor, which is used to get the existing settings and add new
	 * settings.
	 * 
	 * @return the editor
	 */
	final AbstractEditor getEditor() {
		return abstractEditor;
	}

}

