package com.oracle.wci.ucm.sci;

import com.plumtree.remote.ServiceException;
import com.plumtree.remote.sci.IAdminEditor;
import com.plumtree.remote.sci.IAdminPage;
import com.plumtree.remote.sci.TypeNamespace;
import com.plumtree.remote.util.NamedValue;
import com.plumtree.remote.util.NamedValueMap;


/**
 * AdminEditor is the implementation of IAdminEditor registered in
 * server-config&#46;wsdd. In
 * {@link #initialize(String, TypeNamespace, NamedValueMap)}, it creates a
 * {@link DatasourceEditor} or {@link CrawlerEditor} depending on the
 * TypeNamespace passed in. It does not recognize the Card TypeNamespace for
 * remote card submission. For {@link #finalize()} and {@link #getPages()}, it
 * delegates to the DatasourceEditor or CrawlerEditor.
 * 
 * @author <a href="mailto:cdavidso@bea.com">Clinton Davidson</a>
 * @version $Revision: #3 $ ($Change: 348620 $)
 */
public final class AdminEditor implements IAdminEditor {


	/**
	 * Abstract editor, which has common methods for both the DatasourceEditor
	 * and CrawlerEditor.
	 */
	private AbstractEditor abstractEditor;

	/**
	 * Delegates to the abstractEditor.
	 * @param pageInfo incoming named values
	 * @return possibly modified named values
	 * @throws ServiceException required by api
	 * 
	 * @see com.plumtree.remote.sci.IAdminEditor#finalize(com.plumtree.remote.util.NamedValueMap)
	 */
	public NamedValue[] finalize(NamedValueMap pageInfo)
	        throws ServiceException {
		return abstractEditor.finalize(pageInfo);
	}

	/**
	 * Delegates to the abstractEditor.
	 * @return list of pages for datasource or crawler editor
	 * @throws ServiceException required by api
	 * @see com.plumtree.remote.sci.IAdminEditor#getPages()
	 */
	public IAdminPage[] getPages() throws ServiceException {
		return abstractEditor.getPages();
	}

	/**
	 * Uses the TypeNamespace to determine if we are a datasource or crawler,
	 * and instantiates the appropriate editor. Does not support remote card
	 * submission (TypeNamespace.NS_CARD).
	 * @param locale crawler locale, as in 'the majority of documents are in this language'
	 * @param namesp variable which tells us if this is a data source, crawler, or remote submission
	 * @param parentInfo data source info for a crawl; empty for a data source
	 * @throws ServiceException required by api
	 * 
	 * @see com.plumtree.remote.sci.IAdminEditor#initialize(java.lang.String,
	 *      com.plumtree.remote.sci.TypeNamespace,
	 *      com.plumtree.remote.util.NamedValueMap)
	 */
	public void initialize(String locale, TypeNamespace namesp,
	        NamedValueMap parentInfo) throws ServiceException {
		if (namesp.equals(TypeNamespace.NS_DS)) {
			abstractEditor = new DatasourceEditor(locale, parentInfo);
		}
		else if (namesp.equals(TypeNamespace.NS_CRAWL)) {
			abstractEditor = new CrawlerEditor(locale, parentInfo);
		}
		else {
			abstractEditor = null;
		}

	}
	

}

