package com.oracle.wci.ucm.sci;

import oracle.stellent.ridc.IdcClient;
import oracle.stellent.ridc.IdcClientException;
import oracle.stellent.ridc.IdcClientManager;
import oracle.stellent.ridc.IdcContext;
import oracle.stellent.ridc.model.DataBinder;
import oracle.stellent.ridc.protocol.ServiceResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.oracle.wci.ucm.CrawlConstants;
import com.oracle.wci.ucm.UcmConstants;
import com.plumtree.remote.sci.SciLabelElement;
import com.plumtree.remote.sci.SciPage;
import com.plumtree.remote.sci.SciTextElement;
import com.plumtree.remote.sci.TypeStyle;
import com.plumtree.remote.sci.ValidationException;
import com.plumtree.remote.util.NamedValueMap;

/**
 * Queries for RDIC connection information: server, host and username, and
 * validates that the specified server can be pinged.
 * 
 * @author <a href="mailto:clinton.davidson@oracle.com">Clinton Davidson</a>
 * @version $Revision: #3 $ ($Change: 348620 $)
 */
public final class ConnectionInfoPage extends AbstractPage {
	/**
	 * Key to the localized validation message for ucm username.
	 */
	private static final String KEY_USER_VALIDATION_MESSAGE = "sci.connectioninfopage.usernameelement.validationMessage";
	/**
	 * Key to localized label for username.
	 */
	private static final String KEY_USER_LABEL = "sci.connectioninfopage.usernameelement.label";
	/**
	 * Key to localized validation message for ucm server.
	 */
	private static final String KEY_SERVER_VALIDATION_MESSAGE = "sci.connectioninfopage.serverelement.validationMessage";
	/**
	 * Key to localized label for ucm server.
	 */
	private static final String KEY_SERVER_LABEL = "sci.connectioninfopage.serverelement.label";
	/**
	 * Key to localized validation message for idc port.
	 */
	private static final String KEY_PORT_VALIDATION_MESSAGE = "sci.connectioninfopage.portelement.validationMessage";
	/**
	 * Key to localized label for idc port.
	 */
	private static final String KEY_PORT_LABEL = "sci.connectioninfopage.portelement.label";
	/**
	 * Key to localized page title.
	 */
	private static final String KEY_TITLE = "sci.connectioninfopage.title";
	/**
	 * Key to localized page instructions.
	 */
	private static final String KEY_INSTRUCTIONS = "sci.connectioninfopage.instructions";

	/**
	 * Apache Commons Logger.
	 */
	private static final Log LOGGER = LogFactory.getLog(ConnectionInfoPage.class);

	/**
	 * Default constructor- calling the super constructor sets the editor and
	 * locale.
	 * 
	 * @param editor editor used for storing validation error messages.
	 */
	public ConnectionInfoPage(AbstractEditor editor) {
		super(editor);
	}

	/**
	 * Gets three text elements: idc username, idc servername, and idc port.
	 * @param errorCode errorCode thrown in validate, if unable to ping the server using the provided username, servername, and port
	 * @param pageInfo existing username, servername, and port, if this was not the first time run.
	 * @return page with username, servername, and port elements.
	 * @see com.oracle.wci.ucm.sci.AbstractPage#getContent(int,
	 *      com.plumtree.remote.util.NamedValueMap)
	 */
	@Override
	public String getContent(int errorCode, NamedValueMap pageInfo)  {
		final SciPage page = new SciPage();
		if (errorCode > 0){
			final String errorMessage = this.getEditor().getValidationExceptionMessage(errorCode);
			final SciLabelElement labelElement = new SciLabelElement(errorMessage,TypeStyle.STYLE_IMPORTANT);
			page.add(labelElement);
		}

		final SciTextElement serverElement = createServerElement(pageInfo);
		page.add(serverElement);

		final SciTextElement portElement = createPortElement(pageInfo);
		page.add(portElement);

		final SciTextElement usernameElement = createUsernameElement(pageInfo);
		page.add(usernameElement);

		return page.toString();
	}

	/**
	 * Creates a username element.
	 * 
	 * @param pageInfo
	 *            used to add the existing username
	 * @return text element with label, validation message, and existing
	 *         username
	 */
	private SciTextElement createUsernameElement(NamedValueMap pageInfo) {
		final String usernameLabel = this.getEditor().getResourceBundle().getString(KEY_USER_LABEL);
		final SciTextElement usernameElement = new SciTextElement(CrawlConstants.Provider.USERNAME,
				usernameLabel);
		final String username = pageInfo.get(CrawlConstants.Provider.USERNAME);
		if (StringUtils.isNotBlank(username)) {
			usernameElement.setValue(username);
		}
		final String emptyUserValidationMessage = this.getEditor().getResourceBundle().getString(
				KEY_USER_VALIDATION_MESSAGE);
		usernameElement.setMandatoryValidation(emptyUserValidationMessage);
		return usernameElement;
	}

	/**
	 * Creates a server element.
	 * 
	 * @param pageInfo
	 *            used to add the existing server
	 * @return text element with label, validation message, and existing server
	 */
	private SciTextElement createServerElement(NamedValueMap pageInfo) {
		final String usernameLabel = this.getEditor().getResourceBundle().getString(KEY_SERVER_LABEL);
		final SciTextElement usernameElement = new SciTextElement(CrawlConstants.Provider.UCM_SERVER,
				usernameLabel);
		final String username = pageInfo.get(CrawlConstants.Provider.UCM_SERVER);
		if (StringUtils.isNotBlank(username)) {
			usernameElement.setValue(username);
		}
		final String emptyUserValidationMessage = this.getEditor().getResourceBundle().getString(
				KEY_SERVER_VALIDATION_MESSAGE);
		usernameElement.setMandatoryValidation(emptyUserValidationMessage);
		return usernameElement;
	}

	/**
	 * Creates a port element.
	 * 
	 * @param pageInfo
	 *            used to add the existing port
	 * @return text element with label, validation message, and existing port
	 */
	private SciTextElement createPortElement(NamedValueMap pageInfo) {
		final String usernameLabel = this.getEditor().getResourceBundle().getString(KEY_PORT_LABEL);
		final SciTextElement usernameElement = new SciTextElement(CrawlConstants.Provider.UCM_PORT,
				usernameLabel);
		final String username = pageInfo.get(CrawlConstants.Provider.UCM_PORT);
		if (StringUtils.isNotBlank(username)) {
			usernameElement.setValue(username);
		}
		final String emptyUserValidationMessage = this.getEditor().getResourceBundle().getString(
				KEY_PORT_VALIDATION_MESSAGE);
		usernameElement.setMandatoryValidation(emptyUserValidationMessage);
		return usernameElement;
	}

	/**
	 * Gets the localized instructions for 'Enter the UCM server name, port, and crawler username.'.
	 * @return localized instructions
	 * @see com.oracle.wci.ucm.sci.AbstractPage#getInstructions()
	 */
	@Override
	public String getInstructions()  {
		return this.getEditor().getResourceBundle().getString(
				KEY_INSTRUCTIONS);
	}

	 /**
	 * Gets the localized title for 'UCM Connection Information'.
	 * @return localized instrtitleuctions
	 * @see com.oracle.wci.ucm.sci.AbstractPage#getTitle()
	 */
	@Override
	public String getTitle()  {
		return this.getEditor().getResourceBundle().getString(
				KEY_TITLE);
	}

	/**
	 * Uses the provided connection information to ping the server.
	 * @param pageInfo that contains connection information
	 * @throws ValidationException if unable to ping server.
	 * 
	 * @see com.oracle.wci.ucm.sci.AbstractPage#validatePage(com.plumtree.remote.util.NamedValueMap)
	 */
	@Override
	public void validatePage(NamedValueMap pageInfo) throws ValidationException {
		// get the server, port, and user
		final String server = pageInfo.get(CrawlConstants.Provider.UCM_SERVER);
		final String port = pageInfo.get(CrawlConstants.Provider.UCM_PORT);
		final String username = pageInfo.get(CrawlConstants.Provider.USERNAME);
		// construct the idc url
		final String idcUrl = "idc://" + server + ":" + port;
		final IdcClientManager idcManager = new IdcClientManager();
		try {
			final IdcClient idcClient = idcManager.createClient(idcUrl);
			final IdcContext idcContext = new IdcContext(username);
			// Create the DataBinder and populated it with needed information
			// for
			// the service call
			final DataBinder binder = idcClient.createBinder();
			binder.putLocal(UcmConstants.IDC_SERVICE, UcmConstants.Service.PING_SERVER);

			// Join the binder and the user context and perform the service call
			final ServiceResponse response = idcClient.sendRequest(idcContext, binder);

			// Convert the response to a dataBinder
			final DataBinder resultsBinder = response.getResponseAsBinder();

			// Log the status of the service call
			LOGGER.info(String.format("Ping Server response: %s", resultsBinder
					.getLocal("StatusMessage")));

		}
		catch (IdcClientException e) {
			final String errorMessage = e.getMessage();
			//use a random number, as an error code is not returned by ucm
			//todo determine if an error code is returned by ucm
			final int errorCode = 123;
			this.getEditor().addValidationExceptionMessage(errorCode, errorMessage);
			LOGGER.error(errorMessage, e);
			// use a random number
			// perhaps localize the message, or translate it into something
			// usable
			// throw a validation exception
			throw new ValidationException(errorCode);

		}
	}

}
