/*
 * Copyright (c) 2008 BEA Systems, Inc. All Rights Reserved.
 *
 * $Id: //Dev/ucmcrawler/main/src/com/oracle/wci/ucm/sci/ContainerTree.java#3 $
 *
 * This software is the confidential and proprietary information of BEA Systems, Inc. ("Confidential
 * Information"). You shall not disclose such Confidential Information and shall use it only in accordance
 * with the terms of the license agreement you entered into with BEA Systems Inc.
 */

package com.oracle.wci.ucm.sci;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;

import com.plumtree.remote.sci.ITreeIterator;
import com.plumtree.remote.sci.ITreeIteratorNode;
import com.plumtree.remote.util.NamedValueMap;

/**
 * UCM implementation if ITreeIterator.
 * 
 * @author <a href="mailto:clinton.davidson@oracle.com">Clinton Davidson</a>
 * @version $Revision: #3 $ ($Change: 348620 $)
 */
public final class ContainerTree implements ITreeIterator {
	/**
	 * The root container node held in the tree.
	 */
	private ContainerTreeNode rootNode;
	/**
	 * The current container node held in the tree.
	 */
	private ContainerTreeNode currentNode;

	/**
	 * The hash of nodes used to find a node.
	 */
	private HashMap<String, ContainerTreeNode> nodeMap = new HashMap<String, ContainerTreeNode>();

	/**
	 * The name delimiter for container names.
	 */
	private static final String LOCATION_DELIM = "/";

	/**
	 * Gets the children of this node.
	 * @return subfolders
	 * @throws Exception unknown
	 * @see com.plumtree.remote.sci.ITreeIterator#children()
	 */
	public ITreeIteratorNode[] children() throws Exception {
		final ITreeIteratorNode[] nodes = new ITreeIteratorNode[currentNode.getChildren().size()];
		final Iterator childIter = currentNode.getChildren().iterator();
		for (int i = 0; childIter.hasNext(); i++) {
			nodes[i] = (ITreeIteratorNode) childIter.next();
		}
		return nodes;
	}

	/**
	 * Closes this tree.
	 * 
	 * @throws Exception
	 *             if anything goes wrong
	 */
	public void close() throws Exception {
		rootNode = null;
		currentNode = null;
	}

	/**
	 * Gets the id of the current node.
	 * 
	 * @return id of the current node
	 */
	public String getCurrentLocation() {
		return currentNode.getLocation();
	}

	/**
	 * Gets the locale.
	 * 
	 * @return {@code null}.
	 * @see com.plumtree.remote.sci.ITreeIterator#getLocale()
	 */
	public Locale getLocale() {
		return null;
	}

	/**
	 * Gets the name of the current node.
	 * 
	 * @return name of the current node.
	 */
	public String getLocationName() {
		return currentNode.getDisplayName();
	}

	/**
	 * Builds a root node only implementation of the tree.
	 * 
	 * @param namedValueMap
	 *            map used to initialize the iterator with values from the map.
	 * @throws Exception
	 *             if an error occurred when initializing the iterator
	 */
	public void initializeIterator(NamedValueMap namedValueMap) throws Exception {
		rootNode = new ContainerTreeNode(null, namedValueMap.get("Name"), namedValueMap
				.get("ImageName"), namedValueMap.get("Location"));
		currentNode = rootNode;
	}

	/**
	 * Adds a node to the tree.
	 * 
	 * @param name
	 *            node name
	 * @param imageName
	 *            node image (open or closed folder)
	 * @param location
	 *            node location
	 */
	public void addNode(String name, String imageName, String location) {
		final ContainerTreeNode newNode = new ContainerTreeNode(currentNode, name, imageName, location);
		currentNode.insertNode(newNode);
		nodeMap.put(location, newNode);
	}

	/**
	 * Whether the current node is the same as the root node.
	 * 
	 * @return {@code true} if at the root
	 */
	public boolean isAtRoot() {
		return (currentNode.getParent() == null);
	}

	/**
	 * Moves the current node to a specific node in the tree.
	 * 
	 * @param location
	 *            location to move to
	 */
	public void moveTo(String location) {
		final ContainerTreeNode moveNode = nodeMap.get(location);
		if (moveNode != null) {
			currentNode = moveNode;
		}
		else {
			System.out.println("ERROR, could not find node " + location);
		}

	}

	/**
	 * Moves the current node to the parent of the current node.
	 */
	public void moveToParent() {
		if (!isAtRoot()) {
			currentNode = currentNode.getParent();
		}
	}

	/**
	 * Moves the current node to the root node.
	 */
	public void moveToRoot() {
		currentNode = rootNode;
	}

}