/*
 * Copyright (c) 2008 BEA Systems, Inc. All Rights Reserved.
 *
 * $Id: //Dev/ucmcrawler/main/src/com/oracle/wci/ucm/sci/FolderTreePage.java#5 $
 *
 * This software is the confidential and proprietary information of BEA Systems, Inc. ("Confidential
 * Information"). You shall not disclose such Confidential Information and shall use it only in accordance
 * with the terms of the license agreement you entered into with BEA Systems Inc.
 */

package com.oracle.wci.ucm.sci;

import java.util.ArrayList;
import java.util.List;

import oracle.stellent.ridc.IdcClient;
import oracle.stellent.ridc.IdcClientException;
import oracle.stellent.ridc.IdcClientManager;
import oracle.stellent.ridc.IdcContext;
import oracle.stellent.ridc.model.DataBinder;
import oracle.stellent.ridc.model.DataObject;
import oracle.stellent.ridc.model.DataResultSet;
import oracle.stellent.ridc.protocol.ServiceResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.oracle.wci.ucm.CrawlConstants;
import com.oracle.wci.ucm.UcmConstants;
import com.oracle.wci.ucm.UcmConstants.Field;
import com.plumtree.remote.ServiceException;
import com.plumtree.remote.crawler.ChildContainer;
import com.plumtree.remote.crawler.CrawlerConstants;
import com.plumtree.remote.sci.ITreeIterator;
import com.plumtree.remote.sci.SciLabelElement;
import com.plumtree.remote.sci.SciPage;
import com.plumtree.remote.sci.SciSelectElement;
import com.plumtree.remote.sci.SciTreeElement;
import com.plumtree.remote.sci.TypeStorage;
import com.plumtree.remote.sci.TypeStyle;
import com.plumtree.remote.sci.ValidationException;
import com.plumtree.remote.util.NamedValue;
import com.plumtree.remote.util.NamedValueMap;

/**
 * Allows the user to select a folder in the UCM folder structure, and select the crawl depth.
 * @author <a href="mailto:clinton.davidson@oracle.com">Clinton Davidson</a>
 * @version $Revision: #5 $ ($Change: 348667 $)
 */
public final class FolderTreePage extends AbstractPage {
	/**
	 * Trash name- used to not list trash in folder picker.
	 */
	private static final String TRASH = "Trash";
	/**
	 * Key for location in tree named value map.
	 */
	private static final String LOCATION = "Location";
	/**
	 * Key for image name in tree named value map.
	 */
	private static final String IMAGE_NAME = "ImageName";
	/**
	 * Key for folder name in tree named value map.
	 */
	private static final String NAME = "Name";
	/**
	 * Key for localized page title.
	 */
	private static final String KEY_TITLE = "sci.foldertreepage.title";
	/**
	 * Key for localized page instructions.
	 */
	private static final String KEY_INSTRUCTIONS = "sci.foldertreepage.instructions";
	/**
	 * Icon path for a closed folder.
	 */
	private static final String ICON = "plumtree/portal/public/img/icon_folder.gif";
	/**
	 * Icon path for an open folder.
	 */
	private static final String CURRENT_ICON = "plumtree/portal/public/img/icon_folder_open.gif";
	/**
	 * Key to localized value for the depth element validation message.
	 */
	private static final String KEY_DEPTH_VALIDATION_MESSAGE = "sci.foldertreepage.depthelement.validationMessage";
	/**
	 * Key to localized value for the depth element label.
	 */
	private static final String KEY_DEPTH_LABEL = "sci.foldertreepage.depthelement.label";
	/**
	 * Key to localized value for unlimited crawl depth.
	 */
	private static final String KEY_UNLIMITED = "sci.foldertreepage.unlimited";
	/**
	 * Key to localized root folder name.
	 */
	private static final String KEY_ROOT_FOLDER_NAME = "sci.foldertreepage.rootfoldername";
	/**
	 * Key to localized tree no folders message.
	 */
	private static final String KEY_TREE_NO_FOLDERS = "sci.foldertreepage.tree.nofolders.message";
	/**
	 * Key to localized tree help message.
	 */
	private static final String KEY_TREE_NO_SELECTION = "sci.foldertreepage.tree.nofolders.help";
	/**
	 * Apache Commons Logger.
	 */
	private static final Log LOGGER = LogFactory.getLog(FolderTreePage.class);

	/**
	 * Default constructor- calling the super constructor sets the editor and
	 * locale.
	 * 
	 * @param editor
	 *            editor with access to data source values
	 */
	public FolderTreePage(AbstractEditor editor) {
		super(editor);
	}

	/**
	 * Returns a tree element and a crawl depth element.
	 * 
	 * @param errorCode
	 *            ignored, as no validation is done
	 * @param pageInfo
	 *            contains tree repost information, and existing crawl depth
	 * @return tree element and depth element
	 * @throws ServiceException
	 *             never, as a label is added if unable to get information from
	 *             ucm
	 * @see com.oracle.wci.ucm.sci.AbstractPage#getContent(int,
	 *      com.plumtree.remote.util.NamedValueMap)
	 */
	@Override
	public String getContent(int errorCode, NamedValueMap pageInfo) throws ServiceException {
		final SciPage page = new SciPage();
		try {
			final String folderId = getFolderIdFromPageInfo(pageInfo);
			final DataBinder resultsBinder = getCollectionDisplayResponseBinder(folderId);
			final String parentFolderId = getParentIdFromResponseBinder(resultsBinder);
			final String parentFolderName = getParentFolderName(parentFolderId);
			final ChildContainer[] children = getSubFolders(resultsBinder);
			final SciTreeElement sciTree = createFolderTreeElement(pageInfo, parentFolderId,
					parentFolderName, children);
			page.add(sciTree);
		}
		catch (IdcClientException e) {
			final SciLabelElement labelElement = new SciLabelElement(e.getMessage(),
					TypeStyle.STYLE_IMPORTANT);
			page.add(labelElement);
		}

		// element for crawl depth
		final SciSelectElement depthElement = createCrawlDepthElement(pageInfo);
		page.add(depthElement);

		return page.toString();

	}

	/**
	 * Gets the folderId from the page info.
	 * 
	 * @param pageInfo
	 *            page info which contains the crawl root and repost value
	 * @return folder id from repost, if that is not null, then from the crawl
	 *         root if that is not null, then from the initial root.
	 */
	private String getFolderIdFromPageInfo(NamedValueMap pageInfo) {
		String repost = pageInfo.get(CrawlConstants.Provider.REPOST);
		// report should be null if blank
		if (StringUtils.isEmpty(repost)) {
			repost = null;
		}
		// get the root folder from the page info
		final String rootFolderName = this.getEditor().getResourceBundle().getString(
				KEY_ROOT_FOLDER_NAME);
		final String initialRoot = rootFolderName + ":-1";
		String crawlRoot = pageInfo.get(CrawlConstants.Provider.ROOT_FOLDER_NAME_AND_ID);
		if (StringUtils.isEmpty(crawlRoot) || !crawlRoot.contains(":")) {
			crawlRoot = initialRoot;
		}
		int colonPos = crawlRoot.indexOf(':');
		String folderId = crawlRoot.substring(colonPos + 1);
		if (null != repost) {
			colonPos = repost.indexOf(':');
			folderId = repost.substring(colonPos + 1);
		}
		return folderId;
	}

	/**
	 * Create the tree element.
	 * 
	 * @param pageInfo
	 *            page into which contains the selected value and the repost
	 *            location (clicked on folder)
	 * @param parentFolderId
	 *            parent collection id
	 * @param parentFolderName
	 *            parent collection name
	 * @param children
	 *            array of child folders of the current element as
	 *            ChildContainers
	 * @return populated SciTreeElement
	 */
	private SciTreeElement createFolderTreeElement(NamedValueMap pageInfo, String parentFolderId,
			String parentFolderName, ChildContainer[] children) {
		// determine the repost value
		String repost = pageInfo.get(CrawlConstants.Provider.REPOST);
		// report should be null if blank
		if (StringUtils.isEmpty(repost)) {
			repost = null;
		}
		// get the root folder from the page info
		final String rootFolderName = this.getEditor().getResourceBundle().getString(
				KEY_ROOT_FOLDER_NAME);
		final String initialRoot = rootFolderName + ":-1";
		String crawlRoot = pageInfo.get(CrawlConstants.Provider.ROOT_FOLDER_NAME_AND_ID);
		if (StringUtils.isEmpty(crawlRoot) || !crawlRoot.contains(":")) {
			crawlRoot = initialRoot;
		}
		String folderNameAndId = crawlRoot;
		int colonPos = crawlRoot.indexOf(':');
		String folderName = crawlRoot.substring(0, colonPos);
		if (null != repost) {
			colonPos = repost.indexOf(':');
			folderName = repost.substring(0, colonPos);
			folderNameAndId = repost;
		}

		if (StringUtils.isEmpty(repost)) {
			repost = crawlRoot;
		}

		final String parentNameAndId = parentFolderName + ':' + parentFolderId;
		final ITreeIterator treeIter = buildTree(parentFolderName, parentNameAndId, folderName,
				folderNameAndId, children);
		// id is location = name + : + id
		final boolean rootChoosable = false;
		final int numColumns = 2;
		final String noFoldersMessage = getEditor().getResourceBundle().getString(
				KEY_TREE_NO_FOLDERS);
		final String noSelectionMessage = getEditor().getResourceBundle().getString(KEY_TREE_NO_SELECTION);

		/*
		 * treeIterator - iterator for particular directory formName - name of
		 * the field. The formName should be uppercase as the portal will
		 * uppercase all names in the NamedValue[] returned by
		 * IAdminEditor.finalize(). formValue - default field value
		 * repostLocation - location to repost to (e.g., a subfolder) repostRoot -
		 * programmatic name of root location rootDisplayName - display name of
		 * root location chooseRoot - whether or not root is selectable
		 * rootImage - image to display for for root emptyMessage - JavaScript
		 * message to display if there are no Nodes in this 'folder'
		 * noSelectionMessage - JavaScript message to display if nothing is
		 * selected columns - number of columns to display
		 */
		final SciTreeElement sciTree = new SciTreeElement(treeIter,
				CrawlConstants.Provider.ROOT_FOLDER_NAME_AND_ID, // form name
				crawlRoot, // form value
				repost, // repost location
				parentNameAndId, // parentId, // repost root programmatic id
				parentFolderName, // parent, // root display name
				rootChoosable, ICON, noFoldersMessage, noSelectionMessage, numColumns);
		return sciTree;
	}

	/**
	 * Gets the subfolders from the response binder.
	 * 
	 * @param resultsBinder
	 *            DataBinder from call to COLLECTION_DISPLAY for the current
	 *            folder
	 * @return ChildContainer[] of subfolders of the current folder
	 * @throws IdcClientException
	 *             if a ucm error occurred
	 */
	private ChildContainer[] getSubFolders(DataBinder resultsBinder) throws IdcClientException {
		ChildContainer[] children = new ChildContainer[0];

		// get the collections result set and populate child containers
		final DataResultSet resultSet = resultsBinder
				.getResultSet(UcmConstants.ResultSet.COLLECTIONS);
		final List<ChildContainer> childContainerList = new ArrayList<ChildContainer>();
		for (DataObject dataObject : resultSet.getRows()) {
			if (!dataObject.get(Field.D_COLLECTION_MARK).equalsIgnoreCase(TRASH)) {
				final String ucmFolderName = dataObject.get(UcmConstants.Field.D_COLLECTION_NAME);
				final String ucmFolderId = dataObject.get(UcmConstants.Field.D_COLLECTION_ID);
				final ChildContainer container = new ChildContainer();
				container.setDisplayName(ucmFolderName);
				container.setLocation(ucmFolderName + ':' + ucmFolderId);
				childContainerList.add(container);
			}
		}
		children = childContainerList.toArray(new ChildContainer[childContainerList.size()]);
		return children;
	}

	/**
	 * Gets the parent id of the current folder.
	 * 
	 * @param resultsBinder
	 *            DataBinder from call to COLLECTION_DISPLAY for the current
	 *            folder
	 * @return parent collection id
	 * @throws IdcClientException
	 *             if a ucm error occurred
	 */
	private String getParentIdFromResponseBinder(DataBinder resultsBinder)
			throws IdcClientException {
		return resultsBinder.getLocal(UcmConstants.Field.D_PARENT_COLLECTION_ID);
	}

	/**
	 * Gets the parent folder name.
	 * 
	 * @param parentFolderId
	 *            parent folder id
	 * @return returns the localized name for the root folder if the parent id
	 *         is -1, or the parent folder name is "/", otherwise returns the
	 *         parent folder name.
	 * @throws IdcClientException
	 *             if a ucm error occurred
	 */
	private String getParentFolderName(String parentFolderId) throws IdcClientException {
		// if the parent folder id is -1, don't make a call, just return the
		// root folder name
		final String rootFolderName = this.getEditor().getResourceBundle().getString(
				KEY_ROOT_FOLDER_NAME);
		if (parentFolderId.equals(String.valueOf(-1))) {
			return rootFolderName;
		}
		final String username = this.getEditor().getParentInfo().get(
				CrawlConstants.Provider.USERNAME);
		final String idcUrl = this.getEditor().getParentInfo().get(CrawlConstants.Provider.IDC_URL);
		final IdcClientManager idcManager = new IdcClientManager();

		final IdcClient idcClient = idcManager.createClient(idcUrl);
		final IdcContext idcContext = new IdcContext(username);
		final DataBinder binder = idcClient.createBinder();
		binder.putLocal(UcmConstants.IDC_SERVICE, UcmConstants.Service.COLLECTION_DISPLAY);
		binder.putLocal(UcmConstants.Field.D_COLLECTION_ID, parentFolderId);
		binder.putLocal(UcmConstants.Field.HAS_COLLECTION_ID, String.valueOf(1));
		final ServiceResponse response = idcClient.sendRequest(idcContext, binder);
		final DataBinder resultsBinder = response.getResponseAsBinder();
		String parentName = resultsBinder.getLocal(UcmConstants.Field.D_COLLECTION_NAME);
		// sometimes there is no value for dCollectionName, in which case try
		// collectionName
		if (StringUtils.isEmpty(parentName)) {
			parentName = resultsBinder.getLocal(Field.COLLECTION_NAME);

		}
		// if the parent name is /, use ucm root
		if ("/".equals(parentName)) {
			parentName = rootFolderName;
		}
		return parentName;

	}

	/**
	 * Gets the response binder from the call COLLECTION_DISPLAY.
	 * 
	 * @param folderId
	 *            current folder collection id
	 * @return response binder which contains local data (including the parent
	 *         id) and a collections result set, which contains subfolders.
	 * @throws IdcClientException
	 *             if a cum error occurred
	 */
	private DataBinder getCollectionDisplayResponseBinder(String folderId)
			throws IdcClientException {
		final String username = this.getEditor().getParentInfo().get(
				CrawlConstants.Provider.USERNAME);
		final String idcUrl = this.getEditor().getParentInfo().get(CrawlConstants.Provider.IDC_URL);
		final IdcClientManager idcManager = new IdcClientManager();

		final IdcClient idcClient = idcManager.createClient(idcUrl);
		final IdcContext idcContext = new IdcContext(username);

		// Create the DataBinder and populated it with needed information
		final DataBinder binder = idcClient.createBinder();
		// get the list of subfolders in the folder
		binder.putLocal(UcmConstants.IDC_SERVICE, UcmConstants.Service.COLLECTION_DISPLAY);
		binder.putLocal(UcmConstants.Field.D_COLLECTION_ID, folderId);
		binder.putLocal(UcmConstants.Field.HAS_COLLECTION_ID, String.valueOf(1));
		final ServiceResponse response = idcClient.sendRequest(idcContext, binder);
		final DataBinder resultsBinder = response.getResponseAsBinder();
		return resultsBinder;
	}

	/**
	 * Creates an element that lets users determine the depth of the crawl.
	 * 
	 * @param pageInfo
	 *            pageInfo that contains the current depth
	 * @return SciSelectElement for crawl depth
	 */
	private SciSelectElement createCrawlDepthElement(NamedValueMap pageInfo) {
		final String depthLabel = this.getEditor().getResourceBundle().getString(KEY_DEPTH_LABEL);
		final String depthValidation = this.getEditor().getResourceBundle().getString(
				KEY_DEPTH_VALIDATION_MESSAGE);
		final SciSelectElement depthElement = new SciSelectElement(CrawlerConstants.TAG_DEPTH,
				depthLabel);
		depthElement.setStorageType(TypeStorage.STORAGE_INTEGER);
		depthElement.setMandatoryValidation(depthValidation);

		final NamedValue[] depthNamedValues = createDepthNamedValues();
		depthElement.setValue(depthNamedValues);
		final String existingDepth = pageInfo.get(CrawlerConstants.TAG_DEPTH);
		if (StringUtils.isNotEmpty(existingDepth)) {
			depthElement.setSelectedValue(existingDepth);
		}
		return depthElement;
	}

	/**
	 * Creates the options for the crawler depth element.
	 * 
	 * @return NamedValue[] of options, from -1 (unlimited) to 10.
	 */
	private NamedValue[] createDepthNamedValues() {
		// set named values as -1 to 10, with -1 marked as unlimited
		final String unlimited = this.getEditor().getResourceBundle().getString(KEY_UNLIMITED);
		final int depthLimit = 11;
		final NamedValue[] depthNamedValues = new NamedValue[depthLimit + 1];
		for (int i = -1; i < depthLimit; i++) {
			final String value = String.valueOf(i);
			// if -1, use the unlimited label
			NamedValue namedValue;
			if (i == -1) {
				namedValue = new NamedValue(unlimited, value);
			}
			else {
				namedValue = new NamedValue(value, value);
			}
			depthNamedValues[i + 1] = namedValue;
		}
		return depthNamedValues;
	}

	/**
	 * Gets the localized instructions for 'Select the folder to be used as the
	 * starting point of the crawl. Click on a folder link to go to a
	 * subfolder.'.
	 * @return localized instructions
	 * @throws ServiceException never
	 * 
	 * @see com.oracle.wci.ucm.sci.AbstractPage#getInstructions()
	 */
	@Override
	public String getInstructions() throws ServiceException {
		return this.getEditor().getResourceBundle().getString(KEY_INSTRUCTIONS);
	}

	/**
	 * Gets the localized title for 'Crawler Start Folder and Crawl Depth'.
	 * @return localized title
	 * @throws ServiceException never
	 * @see com.oracle.wci.ucm.sci.AbstractPage#getTitle()
	 */
	@Override
	public String getTitle() throws ServiceException {
		return this.getEditor().getResourceBundle().getString(KEY_TITLE);
	}

	/**
	 * No-op implementation of validatePage.
	 * @param pageInfo ignored
	 * @throws ValidationException never as no additional validation is performed.
	 * @throws ServiceException never
	 * @see com.oracle.wci.ucm.sci.AbstractPage#validatePage(com.plumtree.remote.util.NamedValueMap)
	 */
	@Override
	public void validatePage(NamedValueMap pageInfo) throws ValidationException, ServiceException {
		// no-op
	}

	/**
	 * Builds the tree iterator.
	 * 
	 * @param parentName
	 *            parent name
	 * @param parentId
	 *            parent id, which contains both the name and id
	 * @param currentName
	 *            name of the current node
	 * @param currentId
	 *            id of the current node, which contains both the name and id
	 * @param children
	 *            children of the current note
	 * @return constructed tree iterator
	 */
	public ITreeIterator buildTree(String parentName, String parentId, String currentName,
			String currentId, ChildContainer[] children) {

		final NamedValueMap map = new NamedValueMap();
		map.put(NAME, parentName);
		map.put(IMAGE_NAME, ICON);
		map.put(LOCATION, parentId);

		final ContainerTree tree = new ContainerTree();
		try {
			tree.initializeIterator(map);
			// if we are add the root, we don't need to add the node of move to
			// the current id- we just want to move to parent
			if (currentId.equals(parentId)) {
				tree.moveToParent();
			}
			else {
				tree.addNode(currentName, CURRENT_ICON, currentId);
				tree.moveTo(currentId);
			}
			for (int i = 0; i < children.length; i++) {
				final String currFolderName = children[i].getDisplayName();
				if (!currFolderName.equalsIgnoreCase(TRASH)) {
					tree.addNode(currFolderName, ICON, children[i].getLocation());
				}
			}
			tree.moveToParent();
		}
		catch (Exception e) {
			LOGGER.error("Could not build the child nodes of the tree. " + e.getMessage());
		}

		return (ITreeIterator) tree;
	}

}
